/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __QUEUE_ITEM__H__
#define __QUEUE_ITEM__H__

#include <pthread.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "server.h"
#include "sockets.h"
#include "status-item.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define TYPE_QUEUE_ITEM \
	(queue_item_get_type())
#define QUEUE_ITEM(obj) \
	(GTK_CHECK_CAST ((obj), TYPE_QUEUE_ITEM, QueueItem))
#define IS_QUEUE_ITEM(obj) \
	(GTK_CHECK_TYPE ((obj), TYPE_QUEUE_ITEM))

#define QUEUE_ITEM_CLASS(klass) \
	(GTK_CHECK_CLASS_CAST ((klass), TYPE_QUEUE_ITEM, QueueItemClass))
#define IS_QUEUE_ITEM_CLASS(klass) \
	(GTK_CHECK_CLASS_TYPE ((klass), TYPE_QUEUE_ITEM))

typedef struct _QueueItem           QueueItem;
typedef struct _QueueItemClass      QueueItemClass;

struct _QueueItem
{
	StatusItem status;

/*Public Fields*/

/*Private Fields*/  
	server_data *sdata;
	gboolean high_priority;
	pthread_t thread_id;
	gboolean needs_socket;
	PanSocket *sock;
};

struct _QueueItemClass
{
	StatusItemClass parent_class;

/* virtual functions */
	gint (*run)(QueueItem*);
	gchar* (*serialize)(QueueItem*);
};




/* public */

GtkType       queue_item_get_type            (void);

gboolean      queue_item_is_high_priority    (const QueueItem   *item);

gboolean      queue_item_needs_socket        (const QueueItem   *item);

PanSocket*    queue_item_get_socket          (QueueItem         *item);

void          queue_item_set_socket          (QueueItem         *item,
                                              PanSocket         *sock);

gint          queue_item_run                 (QueueItem         *item);

void          queue_item_abort               (QueueItem         *item);

void          queue_item_emit_sockread_err   (QueueItem         *item);

void          queue_item_emit_sockwrite_err  (QueueItem         *item);

void          queue_item_construct           (QueueItem         *item,
                                              server_data       *sdata,
	                                      gboolean           high_priority,
                                              gboolean           needs_socket);

#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __QUEUE_ITEM__H__ */
