/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/*********************
**********************  Includes
*********************/

#include <string.h>

#include <glib.h>
#include <libgnome/gnome-defs.h>
#include <libgnome/gnome-i18n.h>

#include "debug.h"
#include "log.h"
#include "status-item.h"

/*********************
**********************  Defines / Enumerated types
*********************/

enum {
	DESCRIBE,
	PROGRESS_INIT_STEPS,
	PROGRESS_NEXT_STEP,
	PROGRESS_SET_PERCENT,
	PROGRESS_SET_ACTIVITY,
	STATUS,
	ERROR,
	DONE,
	LAST_SIGNAL
};

/*********************
**********************  Macros
*********************/

/*********************
**********************  Structures / Typedefs
*********************/

/*********************
**********************  Private Function Prototypes
*********************/

static void status_item_class_init (StatusItemClass *klass);

static void status_item_init (StatusItem *sock);

static void status_item_destroy (GtkObject* object);

/*********************
**********************  Variables
*********************/

/***********
************  Extern
***********/

/***********
************  Public
***********/

/***********
************  Private
***********/

static guint status_item_signals[LAST_SIGNAL] = { 0 };

static GtkObjectClass *parent_class = NULL;


/*********************
**********************  BEGINNING OF SOURCE
*********************/

/************
*************  PUBLIC ROUTINES
************/

GtkType
status_item_get_type (void)
{
	static GtkType status_item_type = 0;

	if (!status_item_type)
	{
		static const GtkTypeInfo status_item_info =
		{
			"StatusItem",
			sizeof(StatusItem),
			sizeof(StatusItemClass),
			(GtkClassInitFunc) status_item_class_init,
			(GtkObjectInitFunc) status_item_init,
			/* reserved_1 */ NULL,
			/* reserved_2 */ NULL,
			(GtkClassInitFunc) NULL
		};

		status_item_type = gtk_type_unique (
			GTK_TYPE_OBJECT,
			&status_item_info);
	}

	return status_item_type;
}

/*****
******
*****/

typedef gchar* (*GtkSignal_STRING__NONE)(GtkObject* object,
                                         gpointer user_data);
static void
marshal_STRING__NONE(GtkObject *object,
                     GtkSignalFunc func, gpointer func_data, GtkArg *args)
{
	GtkSignal_STRING__NONE rfunc;
	gchar **return_val;
	return_val = GTK_RETLOC_STRING (args[0]);
       	rfunc = (GtkSignal_STRING__NONE) func;
	*return_val = (*rfunc) (object, func_data);
}

static void
status_item_class_init (StatusItemClass *klass)
{
	GtkObjectClass *object_class = (GtkObjectClass*) klass;

	parent_class = gtk_type_class (GTK_TYPE_OBJECT);

	status_item_signals[DESCRIBE] =
		gtk_signal_new ( "describe",
			GTK_RUN_LAST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,describe),
			marshal_STRING__NONE,
			GTK_TYPE_STRING, 0 );

	status_item_signals[PROGRESS_INIT_STEPS] =
		gtk_signal_new ( "progress_init_steps",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,progress_init_steps),
			gtk_marshal_NONE__INT,
			GTK_TYPE_NONE,
			1, GTK_TYPE_INT );

	status_item_signals[PROGRESS_NEXT_STEP] =
		gtk_signal_new ("progress_next_step",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,progress_next_step),
			gtk_marshal_NONE__NONE,
			GTK_TYPE_NONE,
			0 );

	status_item_signals[PROGRESS_SET_PERCENT] =
		gtk_signal_new ("progress_set_percent",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,progress_set_percent),
			gtk_marshal_NONE__INT,
			GTK_TYPE_NONE,
			1, GTK_TYPE_INT );

	status_item_signals[PROGRESS_SET_ACTIVITY] =
		gtk_signal_new ("progress_set_activity",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,progress_set_activity),
			gtk_marshal_NONE__INT,
			GTK_TYPE_NONE,
			1, GTK_TYPE_INT );

	status_item_signals[STATUS] =
		gtk_signal_new ("status",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,status),
			gtk_marshal_NONE__STRING,
			GTK_TYPE_NONE,
			1, GTK_TYPE_STRING );

	status_item_signals[ERROR] =
		gtk_signal_new ("error",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,error),
			gtk_marshal_NONE__STRING,
			GTK_TYPE_NONE,
			1, GTK_TYPE_STRING );

	status_item_signals[DONE] =
		gtk_signal_new ("done",
			GTK_RUN_FIRST,
			object_class->type,
			GTK_SIGNAL_OFFSET(StatusItemClass,done),
			gtk_marshal_NONE__INT,
			GTK_TYPE_NONE,
			1, GTK_TYPE_INT );

        gtk_object_class_add_signals (
		object_class, 
		status_item_signals,
		LAST_SIGNAL);

	object_class->destroy = status_item_destroy;
	klass->describe = NULL; /* pure virtual function */
}

static void
status_item_init (StatusItem *item)
{
	item->abort = FALSE;
	memset ( &item->thread_id, 0, sizeof(pthread_t) );
}

/*****
******
*****/

static void
status_item_destroy (GtkObject* object)
{
	debug(DEBUG_QUEUE, "GtkObject '%p' in status_item_destroy", object);

	g_return_if_fail (object != NULL);
	g_return_if_fail (IS_STATUS_ITEM (object));
	g_return_if_fail (GTK_OBJECT_CONSTRUCTED (object));

	if (GTK_OBJECT_CLASS (parent_class)->destroy)
		(*GTK_OBJECT_CLASS (parent_class)->destroy) (object);
}

/*****
******
*****/

gchar*
status_item_describe (StatusItem *item)
{
	gchar* retval = NULL;

	g_return_val_if_fail (item!=NULL, NULL);
	g_return_val_if_fail (IS_STATUS_ITEM(item), NULL);

	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[DESCRIBE],
	                 &retval);

	return retval;
}


/*****
******
*****/

void
status_item_abort (StatusItem *item)
{
	item->abort = TRUE;
}

/*****
******
*****/

void
status_item_emit_init_steps (StatusItem *item, gint steps)
{
	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[PROGRESS_INIT_STEPS],
	                 steps);
}

void
status_item_emit_next_step (StatusItem *item)
{
	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[PROGRESS_NEXT_STEP]);
}

void
status_item_emit_progress (StatusItem *item, gint out_of_100)
{
	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[PROGRESS_SET_PERCENT],
			 out_of_100);
}

void
status_item_emit_done (StatusItem *item, gint status)
{
	g_message ( "status_item_emit_done status: %d", status); //FIXME: remove
	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[DONE],
			 status);
}

void
status_item_emit_activity (StatusItem *item, gint n)
{
	gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[PROGRESS_SET_ACTIVITY],
			 n);
}
void
status_item_emit_status (StatusItem *item, const gchar *format, ...)
{
	g_return_if_fail (format!=NULL);

	if ( !strchr (format, '%')) {
		gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[STATUS], format);
		debug(DEBUG_QUEUE,format);
	} else {
		gchar *string;
        	va_list args;
		va_start (args, format);
		string = g_strdup_vprintf (format, args);
		va_end (args);
		gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[STATUS], string);
		debug(DEBUG_QUEUE,string);
		g_free(string);
	}
}

void
status_item_emit_error (StatusItem *item, const gchar *fmt, ...)
{
	g_return_if_fail (fmt!=NULL);

	if ( !strchr (fmt, '%')) {
		debug(DEBUG_QUEUE,
			"StatusItem %p has this error: %s", item, fmt);
		gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[ERROR], fmt);
	} else {
		gchar *str;
        	va_list args;
		va_start (args, fmt);
		str = g_strdup_vprintf (fmt, args);
		va_end (args);
		debug(DEBUG_QUEUE,
			"StatusItem %p has this error: %s", item, str);
		gtk_signal_emit (GTK_OBJECT(item),
	                 status_item_signals[ERROR], str);
		g_free(str);
	}
}

void
status_item_emit_sockread_err (StatusItem *item)
{
	gtk_signal_emit (
		GTK_OBJECT(item),
		status_item_signals[ERROR],
		_("Error reading from socket."));
}

void
status_item_emit_sockwrite_err (StatusItem *item)
{
	gtk_signal_emit (
		GTK_OBJECT(item),
		status_item_signals[ERROR],
		_("Error writing to socket."));

}

void
status_item_next_step_gfunc (gpointer gdata,
                            gpointer p_status_item)
{
	status_item_emit_next_step (STATUS_ITEM(p_status_item));
}
