/*
 * Pan - A Newsreader for X
 * Copyright (C) 1999  Pan Development Team (pan@superpimp.org)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

#ifndef __PAN_STATUS_ITEM__H__
#define __PAN_STATUS_ITEM__H__

#include <pthread.h>

#include <glib.h>
#include <gtk/gtk.h>

#include "server.h"
#include "sockets.h"

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define TYPE_STATUS_ITEM \
	(status_item_get_type())
#define STATUS_ITEM(obj) \
	(GTK_CHECK_CAST ((obj), TYPE_STATUS_ITEM, StatusItem))
#define IS_STATUS_ITEM(obj) \
	(GTK_CHECK_TYPE ((obj), TYPE_STATUS_ITEM))

#define STATUS_ITEM_CLASS(klass) \
	(GTK_CHECK_CLASS_CAST ((klass), TYPE_STATUS_ITEM, StatusItemClass))
#define IS_STATUS_ITEM_CLASS(klass) \
	(GTK_CHECK_CLASS_TYPE ((klass), TYPE_STATUS_ITEM))

typedef struct _StatusItem           StatusItem;
typedef struct _StatusItemClass      StatusItemClass;

struct _StatusItem
{
	GtkObject object;

/*Public Fields*/

/*Private Fields*/  
	server_data *sdata;
	gboolean high_priority;
	pthread_t thread_id;
	gboolean needs_socket;
	PanSocket *sock;
	gboolean abort;
};

struct _StatusItemClass
{
	GtkObjectClass parent_class;

/* virtual functions */
	gchar* (*describe)(StatusItem*);

/* public signals */
	void (*progress_init_steps)(StatusItem*, gint steps);
	void (*progress_next_step)(StatusItem*);
	void (*progress_set_percent)(StatusItem*, gint out_of_100);
	void (*progress_set_activity)(StatusItem*, gint n);
	void (*status)(StatusItem*, const gchar*);
	void (*error)(StatusItem*, const gchar*);
	void (*done)(StatusItem*, gint);
};




/* public */

GtkType       status_item_get_type            (void);

void          status_item_abort               (StatusItem    *item);

gchar*        status_item_describe            (StatusItem    *item);

void          status_item_emit_status         (StatusItem    *item,
                                               const gchar   *format,
					       ...);

void          status_item_emit_error          (StatusItem    *item,
                                               const gchar   *format,
					       ...);

void          status_item_emit_sockread_err   (StatusItem         *item);

void          status_item_emit_sockwrite_err  (StatusItem         *item);

void          status_item_emit_progress       (StatusItem    *item,
                                               gint           out_of_100);

void          status_item_emit_init_steps     (StatusItem    *item,
                                               gint           steps);

void          status_item_emit_next_step      (StatusItem    *item);

void          status_item_emit_activity       (StatusItem    *item,
                                               gint           n);

void          status_item_emit_done           (StatusItem    *item,
                                               gint           status);

void          status_item_next_step_gfunc     (gpointer       gdata,
                                               gpointer       p_status_item);


#ifdef __cplusplus
}
#endif /* __cplusplus */


#endif /* __PAN_STATUS_ITEM__H__ */
